#' Creating log events from raw log files.
#'
#' Creating event logs from a data frame of raw log files from a
#' Multi-Touch-Table at the IWM.
#'
#' @param data Data frame of raw log files created with `parse_logfiles()`.
#' See `?parse_logfiles` for more details.
#' @param xmlpath Path to folder where XML definitions of artworks live.
#' @return Data frame.
#' @export
#' @examples
#' # tbd
create_eventlogs <- function(data, xmlpath) {

  if (!lubridate::is.POSIXt(data$date)){
    cat("########## Convertion variable `date` to POSIXct ##########", "\n")
    data$date <- as.POSIXct(data$date)
  }
  data$glossar <- ifelse(data$artwork == "glossar", 1, 0)
  
  # Remove irrelevant events
  dat <- subset(data, !(data$event %in% c("Start Application",
                                        "Show Application")))

  # Create glossar dictionary ##############################################
  cat("\n########## Creating glossar dictionary ##########", "\n")
  artworks <- unique(stats::na.omit(dat$artwork))
  artworks <- artworks[artworks != "glossar"]
  glossar_files <- unique(subset(dat, dat$artwork == "glossar")$popup)
  glossar_dict <- create_glossardict(artworks, glossar_files, path = xmlpath)

  # Add trace variable #####################################################
  cat("\n########## Adding trace variable... ##########", "\n")
  dat1 <- add_trace(dat, glossar_dict)

  # Close events
  cat("\n########## Closing events... ##########", "\n")
  c1 <- close_events(dat1, "move")
  cat("## --> move events closed.", "\n")
  c2 <- close_events(dat1, "flipCard")
  cat("## --> flipCard events closed.", "\n")
  c3 <- close_events(dat1, "openTopic")
  cat("## --> openTopic events closed.", "\n")
  c4 <- close_events(dat1, "openPopup")
  cat("## --> openPopup events closed.", "\n")
  dat2 <- rbind(c1, c2, c3, c4)
  
  dat2 <- dat2[order(dat2$fileId, dat2$date.start, dat2$timeMs.start), ]

  # Remove all events that do not have a `date.start`
  d1 <- nrow(dat2)
  dat2 <- dat2[!is.na(dat2$date.start), ]
  d2 <- nrow(dat2)
  if(d1 > d2) {
    warning(paste0(d1-d2, " lines that do not contain a start event have been removed. This can happen when events span over more than one log file.\n"))
  }

  rownames(dat2) <- NULL

  # Add case variable ######################################################
  cat("\n########## Adding case and eventId variables... ##########", "\n\n")
  dat3 <- add_case(dat2)
  
  # Add event ID ###########################################################
  dat3$eventId <- seq_len(nrow(dat3))
  dat3 <- dat3[, c("fileId", "eventId", "case",
                   "trace", "glossar", "event", "artwork",
                   "date.start", "date.stop", "timeMs.start",
                   "timeMs.stop", "duration", "topicNumber", "popup",
                   "x.start", "y.start", "x.stop", "y.stop",
                   "distance", "scale.start", "scale.stop",
                   "scaleSize", "rotation.start", "rotation.stop",
                   "rotationDegree")]

  # Add trace for move events ##############################################
  cat("\n\n########## Adding trace variable for move events... ##########", "\n")
  dat4 <- add_trace_moves(dat3)

  # Add topics: file names and topics ######################################
  cat("\n########## Adding information about topics... ##########", "\n\n")
  artworks <- unique(dat4$artwork)
  # remove artworks without XML information
  artworks <- artworks[!artworks %in% c("504", "505")]
  topics <- extract_topics(artworks, pattern = paste0(artworks, ".xml"),
                           path = xmlpath)
  
  dat5 <- add_topic(dat4, topics = topics)
  dat5
}

